from abc import ABC, abstractmethod
from typing import List, Union
from cxxheaderparser.types import Parameter, DecoratedType


class StructBackend(ABC):
    @abstractmethod
    def generate(self) -> str:
        pass

    @abstractmethod
    def process_func_ptr_field_with_self(
        self,
        field_name: str,
        params: List[Parameter],
        return_t: DecoratedType,
        doc_string: str,
    ):
        pass

    @abstractmethod
    def process_data_pointer_field(
        self,
        field_name: str,
        type_name: str,
        data_len: Union[str, int],
        doc_string: str,
        deref="",
    ):
        pass

    @abstractmethod
    def process_outer_managed_pointer_field(
        self, field_name: str, field_type_name: str, doc_string: str
    ):
        pass

    @abstractmethod
    def process_outer_managed_pointer_array_field(
        self,
        field_name: str,
        field_type_name: str,
        data_len: Union[str, int],
        doc_string: str,
        deref="",
    ):
        pass

    @abstractmethod
    def process_string_field(self, field_name: str, doc_string: str):
        pass

    @abstractmethod
    def process_string_array_field(self, field_name: str, doc_string: str):
        pass

    @abstractmethod
    def process_simple_field(
        self, field_name: str, field_type_name: str, doc_string: str
    ):
        pass

    @abstractmethod
    def process_pntos_inheritance_field(
        self, field_name: str, field_type_name: str, doc_string: str
    ):
        pass

    @abstractmethod
    def process_class_docstring(self, doc_string: str):
        pass
